
/*

Input:  name_1 = value_1, ..., name_N = value_N 

        name_i              = value_i:

	Scene             = [ s2name_1 = s2value_1, ... ]
        Color[Type]       = Flat, Height, Monochrome, or Rainbow
	Color[Flat]       = [ RGB_1, ... ]
	Color[Monochrome] = [ RGB_1, ... ]
  	Color[Height]     = [ [RGB_1_1, RGB_1_2], ... ]
	Color[Rainbow]    = [ [RGB_1_1, RGB_1_2], ... ]

Side effects:

plotlib::Defaults2d[Color][Type]       = Flat, Height, Monochrome, or Rainbow
plotlib::Defaults2d[Color][Flat]       = [ RGB_1, ... ]
plotlib::Defaults2d[Color][Height]     = [ [RGB_1_1, RGB_1_2], ... ]
plotlib::Defaults2d[Color][Monochrome] = [ RGB_1, ...]
plotlib::Defaults2d[Color][Rainbow]    = [ [RGB_1_1, RGB_1_2], ... ]

plotlib::Defaults3d[Color][Type]       = Flat, Height, Monochrome, or Rainbow
plotlib::Defaults3d[Color][Flat]       = [ RGB_1, ... ]
plotlib::Defaults3d[Color][Height]     = [ [RGB_1_1, RGB_1_2], ... ]
plotlib::Defaults3d[Color][Monochrome] = [ RGB_1, ...]
plotlib::Defaults3d[Color][Rainbow]    = [ [RGB_1_1, RGB_1_2], ... ]

*/

plotlib::Defaults := proc(set, dim)
begin 
  if set then
     plotlib::setDefaults(args(2..args(0)))
  else
     plotlib::getDefaults(args(2..args(0)))
  end_if
end_proc:

plotlib::setDefaults := proc(dim)
  local dn, dv, eq, i, j, l, n, s, t, x; 
begin
  n := args(0);
  if n < 2 then 
     error("At least one argument expected.")
  end_if;

  l := []; 
  t := table();

  for i from 2 to n do

      eq := args(i);

      if type(eq) <> "_equal" then
         error("Equation name = value expected.")
      end_if;
      dn := op(eq, 1); dv := op(eq, 2); 
      case dn
        of hold(Scene) do
	   if not ( domtype(dv) = DOM_LIST or domtype(dv) = DOM_SET or
	            domtype(dv) = DOM_TABLE or domtype(dv) = DOM_ARRAY) then
	      error("".dn.": Illegal specification.")
	   end_if;
           s := {};
           for j from 1 to nops(dv) do
	       x := op(dv, j);
               if type(x) <> "_equal" then
	          error("".dn.": Illegal option")
	       end_if;
               s := s union {x}
	   end_for;
	   l := l . [[dn, s]];
	   break;
        of hold(XRange) do
	of hold(YRange) do
	   if type(dv) = "_range" and 
	      domtype(float(op(dv, 1))) = DOM_FLOAT and
	      domtype(float(op(dv, 2))) = DOM_FLOAT then
              l := l . [[dn, dv]]
	   else
	      error("".dn.": Illegal specification.")
	   end_if;
	   break;
	of hold(RATIO_YX) do
	   if domtype(float(dv)) <> DOM_FLOAT then
	      error("RATIO_YX: Real number expected.")
	   else
	      l := l . [[dn, float(dv)]]
	   end_if;
	   break;
        of Color[Type]       do
        of Color[Flat]       do
        of Color[Height]     do
        of Color[Monochrome] do
        of Color[Rainbow]    do
	   t[op(dn, 2)] := dv;
           break;
	otherwise
	   error("Unknown option.")
     end_case

  end_for;

  plotlib::check_DefaultColor(t);

  unprotect(plotlib);

     for x in l do
         if dim = 2 then
            plotlib::Defaults2d[x[1]] := x[2]
	 else
	    plotlib::Defaults3d[x[1]] := x[2]
	 end_if
     end_for;

     if nops(t) > 0 then
        if dim = 2 then
           plotlib::Defaults2d[Color] := 
	     plotlib::setDefaultColor(plotlib::Defaults2d[Color], t)
        else
	   plotlib::Defaults3d[Color] := 
	     plotlib::setDefaultColor(plotlib::Defaults3d[Color], t)
	end_if
     end_if;

   protect(plotlib);

   null()

end_proc:

plotlib::is_valid_color := proc(c)
  local cf;
begin
  cf := float(c);
  testtype(c, Type::ListOf(Type::NonNegative, 3, 3)) and 
  c[1] <= 1 and c[2] <= 1 and c[3] <= 1
end_proc:


plotlib::check_DefaultColor := proc(d : DOM_TABLE)
   local df;
begin

   if type(d) = "_index" then
      return()
   end_if;
 
   df := d[Type];
   if type(df) <> "_index" then 
      if not (df in {Flat, Height, Monochrome, Rainbow}) then
         error("Type: Illegal specification.")         
      end_if
   end_if;

   plotlib::check_color_list_1(d[Flat],       "Flat");
   plotlib::check_color_list_1(d[Monochrome], "Monochrome");
   plotlib::check_color_list_2(d[Height],     "Height");
   plotlib::check_color_list_2(d[Rainbow],    "Rainbow");
  
end_proc:

plotlib::check_color_list_1 := proc(df, t)
   local c;
begin 
   if  type(df) <> "_index" then
       if domtype(df) = DOM_LIST and nops(df) > 0 then
          for c from 1 to nops(df) do
              if not plotlib::is_valid_color(df[c]) then 
		 error(t.": Illegal specification.")
	      end_if
	  end_for
       else
          error(t.": Illegal specification.")
       end_if
    end_if
end_proc:

plotlib::check_color_list_2 := proc(df, t)
   local c, cf;
begin 
   if  type(df) <> "_index" then
       if domtype(df) = DOM_LIST and nops(df) > 0 then
          for c from 1 to nops(df) do
	      cf := df[c];
	      if not ( domtype(cf) = DOM_LIST and nops(cf) = 2 and
                       plotlib::is_valid_color(cf[1]) and 
		       plotlib::is_valid_color(cf[2]) ) then
		 error(t.": Illegal specification.")
	      end_if
	  end_for
       else
          error(t.": Illegal specification.")
       end_if
    end_if
end_proc:

plotlib::setDefaultColor := proc(td, t)
  local t1;
begin
  t1 := td;
  if type(t[Type]) <> "_index" then
     t1[Type] := t[Type];
  end_if;
  if type(t[Flat]) <> "_index" then
     t1[Flat] := t[Flat];
  end_if;
  if type(t[Monochrome]) <> "_index" then
     t1[Monochrome] := t[Monochrome];
  end_if;
  if type(t[Height]) <> "_index" then
     t1[Height] := t[Height];
  end_if;
  if type(t[Rainbow]) <> "_index" then
     t1[Rainbow] := t[Rainbow];
  end_if;
  return(t1)
end_proc:


plotlib::getDefaults := proc(dim)
  local COLOR, d, dc, dn, i, l, n, new_args;
begin

/*
  COLOR    := table():
  COLOR[2] := table():
  COLOR[3] := table():
*/

  COLOR[2][Type]       := Flat:
  COLOR[2][Flat]       := InternalColorMap:
  COLOR[2][Height]     := [[RGB::Red, RGB::Green]]:
  COLOR[2][Monochrome] := InternalColorMap:
  COLOR[2][Rainbow]    := [[RGB::Red, RGB::Yellow]]:

  COLOR[3][Type]       := Height:
  COLOR[3][Flat]       := [RGB::Red]:
  COLOR[3][Height]     := [[RGB::Yellow, RGB::Red]]:
  COLOR[3][Monochrome] := [RGB::Red]:
  COLOR[3][Rainbow]    := [[RGB::Blue, RGB::Red]]:

  n := args(0);
  if n = 1 then
     new_args := [Scene, XRange, YRange, Color[Type], Color[Flat],
                  Color[Height], Color[Monochrome], Color[Rainbow]];
     if dim = 2 then
        new_args := append(new_args, RATIO_YX)
     end_if;
  else
     new_args := [args(2..args(0))];
  end_if;
  n := nops(new_args);

  if dim = 2 then
     d := plotlib::Defaults2d;
  else
     d := plotlib::Defaults3d;
  end_if;

  l := null();

  for i from 1 to n do

      dn := op(new_args, i);

      case dn
        of hold(Scene)    do
	of hold(XRange)   do
	of hold(YRange)   do
	of hold(RATIO_YX) do
	   l := l, dn = d[dn];
	   break;
        of Color[Type]       do
        of Color[Flat]       do
        of Color[Height]     do
        of Color[Monochrome] do
        of Color[Rainbow]    do
	   dc := d[Color];
	   if type(dc) = "_index" or type(dc[op(dn, 2)]) = "_index" then
              l := l, dn = COLOR[dim][op(dn, 2)]
	   else
	      l := l, dn = dc[op(dn, 2)]
	   end_if;
	   break;
	otherwise
	   error("Unknown option.")
     end_case

  end_for;

  return( l )
end_proc:
